#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/debugfs.h>
#include <linux/jiffies.h>
#include <linux/init.h>
#include <asm/uaccess.h>

MODULE_AUTHOR("Alessandro Rubini");
MODULE_LICENSE("GPL");

/* variables to be traced; for 32bit use jiffies instead */
static u8 var8 = 'x';
static u16 var16 = 16;
static u32 bool = 0;

/* a complete file: return a string, just to do something */
ssize_t bug_read(struct file *file, char __user *buf, size_t count,
		 loff_t *offp)
{
	int off = (int)*offp;
	if (off >= 4) return 0; /* EOF */
	count = min((int)count, 4-off);
	copy_to_user(buf, "bao\n"+off, count);
	*offp += count;
	return count;
}

static struct file_operations bug_fops = {
    .read = bug_read,
};

/* these dentries are used to keep track of our files */
static struct dentry *bug_dir, *bug_8, *bug_16, *bug_32, *bug_b, *bug_f;

int __init bug_init(void)
{
	bug_dir = debugfs_create_dir("bug", NULL /* no parent */);
	if (!bug_dir) return -ENOMEM;
	bug_8  = debugfs_create_u8("var8", 0666, bug_dir, &var8);
	bug_16 = debugfs_create_u16("var16", 0666, bug_dir, &var16);
	bug_32 = debugfs_create_u32("jiffies", 0444, bug_dir,
				    (u32 *)&jiffies);
	bug_b  = debugfs_create_bool("bool", 0666, bug_dir, &bool);
	bug_f  = debugfs_create_file("string", 0444, bug_dir, NULL,
				     &bug_fops);
	return 0;
}

void __exit bug_exit(void)
{
	debugfs_remove(bug_f);
	debugfs_remove(bug_32);
	debugfs_remove(bug_16);
	debugfs_remove(bug_8);
	debugfs_remove(bug_dir);
}

module_init(bug_init);
module_exit(bug_exit);
